#include <PalmOS.h>
#include "FEPTglCS.nr.h"

typedef struct 
	{
	UInt16 On;
	UInt16 Off;
	UInt16 AutoEnbleF;
	UInt16 DblBckLgtF;
	} FEPTglPreferenceType;

void HwrDisplayAttributes(Boolean set, UInt8 kind, void *val)
	SYS_TRAP(sysTrapHwrDisplayAttributes);

#define appFileCreator			'FTgC'
#define appVersionNum			0x02
#define appPrefID				0x01
#define appPrefVersionNum		0x01

#define appLocalID				0x00	//Ftr
#define chkBacklight			0x04	//Ftr

#define Swap16(n) (((((UInt16)n) << 8) & 0xFF00) | ((((UInt16)n) >> 8) & 0x00FF))

static Boolean ChkBacklight(void)
{
	Boolean val;
	HwrDisplayAttributes(false, 0x12, &val);
	return val;
}

static void Deactivate(void)
{
	LocalID dbID;
	FtrGet (appFileCreator, appLocalID, &dbID);
	SysNotifyUnregister(0, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);
	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	MenuEraseStatus (NULL);
}

static void Activate(void)
{
	LocalID dbID;
	FtrGet (appFileCreator, appLocalID, &dbID);
	SysNotifyRegister (0, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);
	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	AlmSetAlarm (0, dbID, NULL, (TimGetSeconds () + 3), NULL);
}

static void SetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (FEPTglPreferenceType), false);
}

static void GetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(FEPTglPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = 0;
			prefs->Off        = 1;
			prefs->AutoEnbleF = 0;
			prefs->DblBckLgtF = 0;

			SetFEPTglPreference(prefs);
		}
}

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyUnregister(cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, sysNotifyNormalPriority);
	FtrUnregister (appFileCreator, appLocalID    );
	FtrUnregister (appFileCreator, chkBacklight  );
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyRegister (cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, NULL, sysNotifyNormalPriority, NULL);

	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);
	FtrSet (appFileCreator, appLocalID, dbID);
	FtrSet (appFileCreator, chkBacklight  , prefs.DblBckLgtF);
}

static void GetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;

	prefs.On         = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ));
																		// !0:ON 0:OFF
	prefs.Off        = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ));
																		// !0:ON 0:OFF
	prefs.AutoEnbleF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ));
																		// !0:ON 0:OFF
	prefs.DblBckLgtF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox));
																		// !0:ON 0:OFF
	SetFEPTglPreference(&prefs);
	
	if (prefs.On != 0)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox), prefs.DblBckLgtF);
}

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyEventDequeuedEvent:
					EventPtr DequedEvent;
					DequedEvent = (EventType*)NtfyPrm->notifyDetailsP;
/*					if (DequedEvent->eType == 0x0100)		//penDownEvent 0x0001
					{
						Coord extentX;
						Coord extentY;
						WinGetDisplayExtent (&extentX, &extentY);
//Add and Mod Ver 2.1
						RectangleType rP;
						WinGetWindowFrameRect (WinGetActiveWindow (), &rP);
						if ((Int16)Swap16(DequedEvent->screenY) <= (Int16)(extentY - rP.topLeft.y - 20))
							Deactivate();
//
					}
*/					if (DequedEvent->eType == 0x0400)		//keyDownEvent 0x0004
					{
						EventType event;
						event.eType = keyDownEvent;
						event.data.keyDown.chr = Swap16(DequedEvent->data.keyDown.chr);
						event.data.keyDown.keyCode = Swap16(DequedEvent->data.keyDown.keyCode);
						event.data.keyDown.modifiers = Swap16(DequedEvent->data.keyDown.modifiers);
						if (EvtKeydownIsVirtual (&event) == true)
							Deactivate();

						switch (DequedEvent->data.keyDown.chr)
						{
//Chng Ver 2.2
						case 0x2000:	//chrSpace 0x0020
							DequedEvent->data.keyDown.chr = 0x1801;	//vchrTsm1 0x0118 Henkan
							DequedEvent->data.keyDown.modifiers = 0x0800;
							Deactivate();
							break;

//Chng Ver 2.2
						case 0x0a00:	//chrLineFeed 0x000a
							DequedEvent->data.keyDown.chr = 0x1a01;	//vchrTsm3 0x011a HiraKata
							DequedEvent->data.keyDown.modifiers = 0x0800;
							Deactivate();
							break;

//Chng Ver 2.2
						case 0x0601:	//vchrCommand 0x0106
							DequedEvent->data.keyDown.chr = 0x1b01;	//vchrTsm4 0x011b Nichiei
							DequedEvent->data.keyDown.modifiers = 0x0800;
							Deactivate();
							break;

//Chng Ver 2.2
						case 0x0800:	//chrBackspace 0x0008
							DequedEvent->data.keyDown.chr = 0x0801;	//vchrLaunch 0x0108 Launch
							DequedEvent->data.keyDown.modifiers = 0x0800;
							Deactivate();
							break;
						}
					}
//Add Ver 2.1
					if (DequedEvent->eType == 0x1500)		//menuEvent 0x0015
					{
						Deactivate();
					}
//
					break;

				case sysNotifyMenuCmdBarOpenEvent:
					UInt32 DblBckLgtF;
					FtrGet (appFileCreator, chkBacklight, &DblBckLgtF);
					if (DblBckLgtF != 0)
						if (ChkBacklight() == false)
							break;
					Activate();
					break;
				}
			break;

		case sysAppLaunchCmdAlarmTriggered:
			Deactivate ();
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue(frmP);				 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			FEPTglPreferenceType prefs;				//Check Auto Enable
			GetFEPTglPreference(&prefs);
			if (prefs.On != 0)
			{
				if (prefs.AutoEnbleF != 0)
					Enable();						//activate
				else
				{
					prefs.On  = 0;
					prefs.Off = 1;
					SetFEPTglPreference(&prefs);
				}
			}
			break;

		default:
			break;

		}
	
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
